﻿namespace U2UConsult.Composite.Communication.Sample.AggregatedEvents
{
    using System.ComponentModel;
    using System.ComponentModel.Composition;
    using Microsoft.Practices.Prism.Events;

    /// <summary>
    /// Subscriber using Prism even aggregator.
    /// </summary>
    [Export]
    public class Subscriber : INotifyPropertyChanged
    {
        /// <summary>
        /// Output message.
        /// </summary>
        private string message;

        /// <summary>
        /// Prism event aggregator.
        /// </summary>
        private EventAggregator eventAggregator;

        /// <summary>
        /// Initializes a new instance of the Subscriber class.
        /// </summary>
        public Subscriber()
        {
            Subscriber.InstancesCount++;
        }

        /// <summary>
        /// Finalizes an instance of the Subscriber class. 
        /// </summary>
        ~Subscriber()
        {
            Subscriber.InstancesCount--;
        }

        /// <summary>
        /// Property change notification.
        /// </summary>
        public event PropertyChangedEventHandler PropertyChanged;

        /// <summary>
        /// Gets or sets the instance counter.
        /// </summary>
        public static int InstancesCount { get; set; }

        /// <summary>
        /// Gets or sets the Prism event aggregator.
        /// </summary>
        [Import]
        public EventAggregator EventAggregator
        {
            get { return this.eventAggregator; }
            set { this.eventAggregator = value; }
        }

        /// <summary>
        /// Sets the publisher.
        /// </summary>
        [Import]
        public IPublisher Publisher
        {
            set
            {
                if (value != null)
                {
                    PublishedEvent pe = this.eventAggregator.GetEvent<PublishedEvent>();
                    pe.Subscribe(o => this.Publisher_Published(o));
                    
                    // Apply filter
                    // pe.Subscribe(
                    //    o => this.Publisher_Published(o),
                    //    ThreadOption.PublisherThread,
                    //    false,
                    //    o => DateTime.Now.Minute % 2 == 0);

                    // Use strong reference
                    // pe.Subscribe(
                    //    o => this.Publisher_Published(o),
                    //    true);
                }
            }
        }

        /// <summary>
        /// Gets or sets the output message. 
        /// </summary>
        public string Message
        {
            get
            {
                return this.message;
            }

            set
            {
                this.message = value;
                this.PropertyChanged.Raise(this, o => o.Message);
            }
        }

        /// <summary>
        /// Event handler for the published event.
        /// </summary>
        /// <param name="o">Ignore parameter.</param>
        private void Publisher_Published(object o)
        {
            this.Message = "Publisher published";
        }
    }
}
