﻿namespace U2UConsult.Composite.Communication.Sample.RegularEvents
{
    using System;
    using System.ComponentModel;
    using System.ComponentModel.Composition;

    /// <summary>
    /// Subscriber using regular event.
    /// </summary>
    [Export]
    public class Subscriber : INotifyPropertyChanged
    {
        /// <summary>
        /// Output message.
        /// </summary>
        private string message;

        /// <summary>
        /// Initializes a new instance of the Subscriber class.
        /// </summary>
        public Subscriber()
        {
            Subscriber.InstancesCount++;
        }

        /// <summary>
        /// Finalizes an instance of the Subscriber class.
        /// </summary>
        ~Subscriber()
        {
            Subscriber.InstancesCount--;
        }

        /// <summary>
        /// Property change notification.
        /// </summary>
        public event PropertyChangedEventHandler PropertyChanged;

        /// <summary>
        /// Gets or sets the instance counter.
        /// </summary>
        public static int InstancesCount { get; set; }

        // private IPublisher publisher;

        /// <summary>
        /// Sets the publisher.
        /// </summary>
        [Import]
        public IPublisher Publisher
        {
            // get
            // {
            //     return publisher;
            // }
            
            set
            {
                // if (this.publisher != null)
                // {
                //     this.publisher.Published -= this.Publisher_Published;   
                // }

                // this.publisher = value;
                
                if (value != null)
                {
                    value.Published += this.Publisher_Published;
                }
            }
        }

        /// <summary>
        /// Gets or sets the output message.
        /// </summary>
        public string Message
        {
            get 
            { 
                return this.message; 
            }

            set 
            { 
                this.message = value;
                this.PropertyChanged.Raise(this, o => o.Message);
            }
        }
        
        /// <summary>
        /// Eventhandler for publish event.
        /// </summary>
        /// <param name="sender">Sender of the event: publisher.</param>
        /// <param name="e">Event arguments.</param>
        private void Publisher_Published(object sender, EventArgs e)
        {
            this.Message = "Publisher published";
        }
    }
}
